/*******************************************************************************
File:			Serial.c

Abstract:	Serial I/O function library

Platform:	ATmega8

Functions:	extern void	SerInit(void)
				extern void	SerSendByte(unsigned char chr, unsigned char port)
				extern void SerSendString(char *address, unsigned char port)
				extern void SerProcesses(void)
				extern void	SerSoftUART(void)
				SIGNAL(SIG_UART_RECV)
				SIGNAL(SIG_UART_TRANS)

Version:		1.00	12/08/08	GND	Gary N. Dion

*******************************************************************************/

// OS headers
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

// App required include files
#include "Serial.h"

// Local Variables (Note: Special case with 256 bytes per buffer)
unsigned char in0buf[256];					// USART input buffer
unsigned char in0head = 0;					// USART input buffer head pointer
unsigned char in0tail = 0;					// USART input buffer tail pointer
unsigned char out0buf[256];				// USART output buffer
unsigned char out0head = 0;				// USART output buffer head pointer
unsigned char out0tail = 0;				// USART output buffer tail pointer


/******************************************************************************/
extern void	SerInit(void)
/*******************************************************************************
* ABSTRACT:	This function initializes the serial USART.
*				It sets up the baud rate, data and stop bits, and parity.
*
* INPUT:		None
* RETURN:	Void
*/
{
	// Set baud rate
	UBRRH = 0;
	UBRRL = 7;  								// Equates to 115.2k baud at 14.7456 MHz
//	UBRRL = 23;  								// Equates to 38.4k baud at 14.7456 MHz
//	UBRRL = 47;  								// Equates to 19.2k baud at 14.7456 MHz
//	UBRRL = 95;  								// Equates to 9600 baud at 14.7456 MHz
//	UBRRL = 191;  								// Equates to 4800 baud at 14.7456 MHz

	// Set frame format to 8 data bits, no parity, and 1 stop bit
	UCSRC = (1<<URSEL)|(3<<UCSZ0);

	// Enable Receiver Interrupt, Receiver, and Transmitter
	UCSRB = (1<<RXCIE)|(1<<TXCIE)|(1<<RXEN)|(1<<TXEN);

	return;

}		// End SerInit(...)


/******************************************************************************/
extern void	SerSendByte(unsigned char chr)
/*******************************************************************************
* ABSTRACT:	This function pushes a new character into the output buffer
*				then pre-advances the pointer to the next empty location.
*
* INPUT:		chr	byte to send
*				port	0 is the USART, 1 is the Software UART
* RETURN:	Void
*/
{
	cli();
	out0buf[++out0head] = chr;		 	// Transfer the byte to output buffer

	if (UCSRA & (1<<UDRE))				// If the transmit buffer is empty
	{
		UDR = out0buf[++out0tail];		// Place the byte in the output buffer
	}
	sei();

	return;

}		// End SendByte(...)


/******************************************************************************/
extern char	SerGetByte(void)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string to the serial port.
*
* INPUT:		*szString	Pointer to string to send
*				port	0 is the USART, 1 is the Software UART
* RETURN:	Void
*/
{
	if (in0tail != in0head)					// If incoming bytes are pending
	{
		return in0buf[++in0tail];
	}
	else	
	{
		return 0;
	}

}		// End SerGetByte(...)


/******************************************************************************/
extern void SerSendString(char *address)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string to the serial port.
*
* INPUT:		*szString	Pointer to string to send
*				port	0 is the USART, 1 is the Software UART
* RETURN:	Void
*/
{
	while (*address != 0)				// While not end of string
	{
		SerSendByte(*(address++));	// Send the byte and increment index
	}

	return;

}		// End SendString(...)


/******************************************************************************/
extern void SerProcesses(void)
/*******************************************************************************
* ABSTRACT:	Called by main.c during idle time. Processes any waiting serial
*				characters coming in or going out both serial ports.
*
* INPUT:		None
* RETURN:	Void
*/
{
	char temp;
	char new = 0;

	if (in0tail != in0head)					// If incoming bytes are pending
	{
		temp = in0buf[++in0tail];
		new = 1;
	}
	
	if (new)
	{
		SerSendByte(temp);					// Echo character for debug
//		MsgHandler0(temp);					// And pass it to a handler
	}

	return;

}		// End Serial_Processes(...)


/******************************************************************************/
ISR(SIG_UART_RECV)
/*******************************************************************************
* ABSTRACT:	Called by the receive ISR (interrupt). Saves the next serial byte
*				to the head of the RX buffer.
*/
{
	in0buf[++in0head] = UDR;	  			// Transfer the byte to the input buffer

}		// End SIGNAL(SIG_UART_RECV)


/******************************************************************************/
ISR(SIG_UART_TRANS)
/*******************************************************************************
* ABSTRACT:	Called by the transmit ISR (interrupt). Puts the next serial byte
*				into the TX register.
*/
{
	if (out0tail != out0head)				// If there are outgoing bytes pending
	{
		UDR = out0buf[++out0tail];			// Place the byte in the output buffer
	}

}		// End SIGNAL(SIG_UART_TRANS)
